@@ -41,6 +41,8 @@ import time
 import formats
 import media
 import utils
+# Use same gettext translations
+from mainapp import _
 
 
 # Debugging flag (calls utils.debug_time at the start of every function)
@@ -92,6 +94,9 @@ class TidyManager(threading.Thread):
 
             del_thumb_flag: True if all thumbnail files should be deleted
 
+            del_webp_flag: True if all thumbnail files in .webp or malformed
+                .jpg format should be deleted (see comments below)
+
             del_archive_flag: True if all youtube-dl archive files should be
                 deleted
 
@@ -104,7 +109,7 @@ class TidyManager(threading.Thread):
     def __init__(self, app_obj, choices_dict):
 
         if DEBUG_FUNC_FLAG:
-            utils.debug_time('top 107 __init__')
+            utils.debug_time('top 112 __init__')
 
         super(TidyManager, self).__init__()
 
@@ -154,6 +159,13 @@ class TidyManager(threading.Thread):
         self.del_xml_flag = choices_dict['del_xml_flag']
         # True if all thumbnail files should be deleted
         self.del_thumb_flag = choices_dict['del_thumb_flag']
+        # v2.1.027. In June 2020, YouTube started serving .webp thumbnails.
+        #   At the time of writing, Gtk can't display them. A youtube-dl fix is
+        #   expected, which will convert .webp thumbnails to .jpg; in
+        #   anticipation of that, we add an option to remove .webp files
+        # True if all thumbnail files in .webp or malformed .jpg format should
+        #   be deleted
+        self.del_webp_flag = choices_dict['del_webp_flag']
         # True if all youtube-dl archive files should be deleted
         self.del_archive_flag = choices_dict['del_archive_flag']
 
@@ -175,6 +187,7 @@ class TidyManager(threading.Thread):
         self.json_deleted_count = 0
         self.xml_deleted_count = 0
         self.thumb_deleted_count = 0
+        self.webp_deleted_count = 0
         self.archive_deleted_count = 0
 
 
@@ -204,13 +217,13 @@ class TidyManager(threading.Thread):
         """
 
         if DEBUG_FUNC_FLAG:
-            utils.debug_time('top 207 run')
+            utils.debug_time('top 220 run')
 
         # Show information about the tidy operation in the Output Tab
         if not self.init_obj:
             self.app_obj.main_win_obj.output_tab_write_stdout(
                 1,
-                'Starting tidy operation, tidying up whole data directory',
+                _('Starting tidy operation, tidying up whole data directory'),
             )
 
         else:
@@ -219,112 +232,123 @@ class TidyManager(threading.Thread):
 
             self.app_obj.main_win_obj.output_tab_write_stdout(
                 1,
-                'Starting tidy operation, tidying up ' + media_type \
-                + ' \'' + self.init_obj.name + '\'',
+                _('Starting tidy operation, tidying up \'{0}\'').format(
+                    self.init_obj.name,
+                )
             )
 
         if self.corrupt_flag:
-            text = 'YES'
+            text = _('YES')
         else:
-            text = 'NO'
+            text = _('NO')
 
         self.app_obj.main_win_obj.output_tab_write_stdout(
             1,
-            '   Check videos are not corrupted:   ' + text,
+            '   ' + _('Check videos are not corrupted:') + ' ' + text,
         )
 
         if self.corrupt_flag:
 
             if self.del_corrupt_flag:
-                text = 'YES'
+                text = _('YES')
             else:
-                text = 'NO'
+                text = _('NO')
 
             self.app_obj.main_win_obj.output_tab_write_stdout(
                 1,
-                '   Delete corrupted videos:          ' + text,
+                '   ' + _('Delete corrupted videos:') + ' ' + text,
             )
 
         if self.exist_flag:
-            text = 'YES'
+            text = _('YES')
         else:
-            text = 'NO'
+            text = _('NO')
 
         self.app_obj.main_win_obj.output_tab_write_stdout(
             1,
-            '   Check videos do/don\'t exist:      ' + text,
+            '   ' + _('Check videos do/don\'t exist:') + ' ' + text,
         )
 
         if self.del_video_flag:
-            text = 'YES'
+            text = _('YES')
         else:
-            text = 'NO'
+            text = _('NO')
 
         self.app_obj.main_win_obj.output_tab_write_stdout(
             1,
-            '   Delete all video files:           ' + text,
+            '   ' + _('Delete all video files:') + ' ' + text,
         )
 
         if self.del_video_flag:
 
             if self.del_others_flag:
-                text = 'YES'
+                text = _('YES')
             else:
-                text = 'NO'
+                text = _('NO')
 
             self.app_obj.main_win_obj.output_tab_write_stdout(
                 1,
-                '   Delete other video/audio files:   ' + text,
+                '   ' + _('Delete other video/audio files:') + ' ' + text,
             )
 
         if self.del_descrip_flag:
-            text = 'YES'
+            text = _('YES')
         else:
-            text = 'NO'
+            text = _('NO')
 
         self.app_obj.main_win_obj.output_tab_write_stdout(
             1,
-            '   Delete all description files:     ' + text,
+            '   ' + _('Delete all description files:') + ' ' + text,
         )
 
         if self.del_json_flag:
-            text = 'YES'
+            text = _('YES')
         else:
-            text = 'NO'
+            text = _('NO')
 
         self.app_obj.main_win_obj.output_tab_write_stdout(
             1,
-            '   Delete all metadata (JSON) files: ' + text,
+            '   ' + _('Delete all metadata (JSON) files:') + ' ' + text,
         )
 
         if self.del_xml_flag:
-            text = 'YES'
+            text = _('YES')
         else:
-            text = 'NO'
+            text = _('NO')
 
         self.app_obj.main_win_obj.output_tab_write_stdout(
             1,
-            '   Delete all annotation files:      ' + text,
+            '   ' + _('Delete all annotation files:') + ' ' + text,
         )
 
         if self.del_thumb_flag:
-            text = 'YES'
+            text = _('YES')
         else:
-            text = 'NO'
+            text = _('NO')
 
         self.app_obj.main_win_obj.output_tab_write_stdout(
             1,
-            '   Delete all thumbnail files:       ' + text,
+            '   ' + _('Delete all thumbnail files:') + ' ' + text,
+        )
+
+        if self.del_webp_flag:
+            text = _('YES')
+        else:
+            text = _('NO')
+
+        self.app_obj.main_win_obj.output_tab_write_stdout(
+            1,
+            '   ' + _('Delete .webp/malformed .jpg files:') + ' ' + text,
         )
 
         if self.del_archive_flag:
-            text = 'YES'
+            text = _('YES')
         else:
-            text = 'NO'
+            text = _('NO')
 
         self.app_obj.main_win_obj.output_tab_write_stdout(
             1,
-            '   Delete youtube-dl archive files:  ' + text,
+            '   ' + _('Delete youtube-dl archive files:') + ' ' + text,
         )
 
         # Compile a list of channels, playlists and folders to tidy up (each
@@ -360,20 +384,20 @@ class TidyManager(threading.Thread):
         # Show a confirmation in the Output Tab
         self.app_obj.main_win_obj.output_tab_write_stdout(
             1,
-            'Tidy operation finished',
+            _('Tidy operation finished'),
         )
 
         if self.corrupt_flag:
 
             self.app_obj.main_win_obj.output_tab_write_stdout(
                 1,
-                '   Corrupted videos found:                     ' \
+                '   ' + _('Corrupted videos found:') + ' ' \
                 + str(self.video_corrupt_count),
             )
 
             self.app_obj.main_win_obj.output_tab_write_stdout(
                 1,
-                '   Corrupted videos deleted:                   ' \
+                '   ' + _('Corrupted videos deleted:') + ' ' \
                 + str(self.video_corrupt_deleted_count),
             )
 
@@ -381,13 +405,13 @@ class TidyManager(threading.Thread):
 
             self.app_obj.main_win_obj.output_tab_write_stdout(
                 1,
-                '   New video files detected:                   ' \
+                '   ' + _('New video files detected:') + ' ' \
                 + str(self.video_exist_count),
             )
 
             self.app_obj.main_win_obj.output_tab_write_stdout(
                 1,
-                '   Missing video files detected:               ' \
+                '   ' + _('Missing video files detected:') + ' ' \
                 + str(self.video_no_exist_count),
             )
 
@@ -395,13 +419,13 @@ class TidyManager(threading.Thread):
 
             self.app_obj.main_win_obj.output_tab_write_stdout(
                 1,
-                '   Non-corrupted video files deleted:          ' \
+                '   ' + _('Non-corrupted video files deleted:') + ' ' \
                 + str(self.video_deleted_count),
             )
 
             self.app_obj.main_win_obj.output_tab_write_stdout(
                 1,
-                '   Other video/audio files deleted:            ' \
+                '   ' + _('Other video/audio files deleted:') + ' ' \
                 + str(self.other_deleted_count),
             )
 
@@ -409,7 +433,7 @@ class TidyManager(threading.Thread):
 
             self.app_obj.main_win_obj.output_tab_write_stdout(
                 1,
-                '   Description files deleted:                  ' \
+                '   ' + _('Description files deleted:') + ' ' \
                 + str(self.descrip_deleted_count),
             )
 
@@ -417,7 +441,7 @@ class TidyManager(threading.Thread):
 
             self.app_obj.main_win_obj.output_tab_write_stdout(
                 1,
-                '   Metadata (JSON) files deleted:              ' \
+                '   ' + _('Metadata (JSON) files deleted:') + ' ' \
                 + str(self.json_deleted_count),
             )
 
@@ -425,7 +449,7 @@ class TidyManager(threading.Thread):
 
             self.app_obj.main_win_obj.output_tab_write_stdout(
                 1,
-                '   Annotation files deleted:                   ' \
+                '   ' + _('Annotation files deleted:') + ' ' \
                 + str(self.xml_deleted_count),
             )
 
@@ -433,15 +457,23 @@ class TidyManager(threading.Thread):
 
             self.app_obj.main_win_obj.output_tab_write_stdout(
                 1,
-                '   Thumbnail files deleted:                    ' \
+                '   ' + _('Thumbnail files deleted:') + ' ' \
                 + str(self.thumb_deleted_count),
             )
 
+        if self.del_webp_flag:
+
+            self.app_obj.main_win_obj.output_tab_write_stdout(
+                1,
+                '   ' + _('.webp/malformed .jpg files deleted:') + ' ' \
+                + str(self.webp_deleted_count),
+            )
+
         if self.del_archive_flag:
 
             self.app_obj.main_win_obj.output_tab_write_stdout(
                 1,
-                '   youtube-dl archive files deleted:           ' \
+                '   ' + _('youtube-dl archive files deleted:') + ' ' \
                 + str(self.archive_deleted_count),
             )
 
@@ -467,7 +499,7 @@ class TidyManager(threading.Thread):
         """
 
         if DEBUG_FUNC_FLAG:
-            utils.debug_time('top 470 tidy_directory')
+            utils.debug_time('top 502 tidy_directory')
 
         # Update the main window's progress bar
         self.job_count += 1
@@ -483,7 +515,7 @@ class TidyManager(threading.Thread):
 
         self.app_obj.main_win_obj.output_tab_write_stdout(
             1,
-            'Checking ' + media_type + ' \'' + media_data_obj.name + '\'',
+            _('Checking:') + ' \'' + media_data_obj.name + '\'',
         )
 
         if self.corrupt_flag:
@@ -507,6 +539,9 @@ class TidyManager(threading.Thread):
         if self.del_thumb_flag:
             self.delete_thumb(media_data_obj)
 
+        if self.del_webp_flag:
+            self.delete_webp(media_data_obj)
+
         if self.del_archive_flag:
             self.delete_archive(media_data_obj)
 
@@ -526,7 +561,7 @@ class TidyManager(threading.Thread):
         """
 
         if DEBUG_FUNC_FLAG:
-            utils.debug_time('top 529 check_video_corrupt')
+            utils.debug_time('top 564 check_video_corrupt')
 
         for video_obj in media_data_obj.compile_all_videos( [] ):
 
@@ -567,8 +602,9 @@ class TidyManager(threading.Thread):
 
                             self.app_obj.main_win_obj.output_tab_write_stdout(
                                 1,
-                                '   Deleted (possibly) corrupted video'
-                                + ' file: \'' + video_obj.name + '\'',
+                                '   ' + _(
+                                    'Deleted (possibly) corrupted video file:',
+                                ) + ' \'' + video_obj.name + '\'',
                             )
 
                             self.app_obj.mark_video_downloaded(
@@ -581,8 +617,9 @@ class TidyManager(threading.Thread):
                             # Don't delete it
                             self.app_obj.main_win_obj.output_tab_write_stdout(
                                 1,
-                                '   Video file might be corrupt: \'' \
-                                + video_obj.name + '\'',
+                                '   ' + _(
+                                    'Video file might be corrupt:',
+                                ) + ' \'' + video_obj.name + '\'',
                             )
 
 
@@ -602,7 +639,7 @@ class TidyManager(threading.Thread):
         """
 
         if DEBUG_FUNC_FLAG:
-            utils.debug_time('top 605 check_videos_exist')
+            utils.debug_time('top 642 check_videos_exist')
 
         for video_obj in media_data_obj.compile_all_videos( [] ):
 
@@ -624,7 +661,9 @@ class TidyManager(threading.Thread):
 
                     self.app_obj.main_win_obj.output_tab_write_stdout(
                         1,
-                        '   Video file exists: \'' + video_obj.name + '\'',
+                        '   ' + _(
+                            'Video file exists:',
+                        ) + ' \'' + video_obj.name + '\'',
                     )
 
                 elif video_obj.dl_flag \
@@ -640,8 +679,9 @@ class TidyManager(threading.Thread):
 
                     self.app_obj.main_win_obj.output_tab_write_stdout(
                         1,
-                        '   Video file doesn\'t exist: \'' + video_obj.name \
-                        + '\'',
+                        '   ' + _(
+                            'Video file doesn\'t exist:',
+                        ) + ' \'' + video_obj.name + '\'',
                     )
 
 
@@ -660,7 +700,7 @@ class TidyManager(threading.Thread):
         """
 
         if DEBUG_FUNC_FLAG:
-            utils.debug_time('top 663 delete_video')
+            utils.debug_time('top 703 delete_video')
 
         ext_list = formats.VIDEO_FORMAT_LIST.copy()
         ext_list.extend(formats.AUDIO_FORMAT_LIST)
@@ -759,7 +799,7 @@ class TidyManager(threading.Thread):
         """
 
         if DEBUG_FUNC_FLAG:
-            utils.debug_time('top 762 delete_descrip')
+            utils.debug_time('top 802 delete_descrip')
 
         for video_obj in media_data_obj.compile_all_videos( [] ):
 
@@ -804,7 +844,7 @@ class TidyManager(threading.Thread):
         """
 
         if DEBUG_FUNC_FLAG:
-            utils.debug_time('top 807 delete_json')
+            utils.debug_time('top 847 delete_json')
 
         for video_obj in media_data_obj.compile_all_videos( [] ):
 
@@ -849,7 +889,7 @@ class TidyManager(threading.Thread):
         """
 
         if DEBUG_FUNC_FLAG:
-            utils.debug_time('top 852 delete_xml')
+            utils.debug_time('top 892 delete_xml')
 
         for video_obj in media_data_obj.compile_all_videos( [] ):
 
@@ -894,7 +934,7 @@ class TidyManager(threading.Thread):
         """
 
         if DEBUG_FUNC_FLAG:
-            utils.debug_time('top 897 delete_thumb')
+            utils.debug_time('top 937 delete_thumb')
 
         for video_obj in media_data_obj.compile_all_videos( [] ):
 
@@ -924,6 +964,52 @@ class TidyManager(threading.Thread):
                     self.thumb_deleted_count += 1
 
 
+    def delete_webp(self, media_data_obj):
+
+        """Called by self.tidy_directory().
+
+        Checks all child videos of the specified media data object. If the
+        associated thumbnail file in a .webp or malformed .jpg format exists,
+        delete it
+
+        Args:
+
+            media_data_obj (media.Channel, media.Playlist or media.Folder):
+                The media data object whose directory must be tidied up
+
+        """
+
+        if DEBUG_FUNC_FLAG:
+            utils.debug_time('top 983 delete_webp')
+
+        for video_obj in media_data_obj.compile_all_videos( [] ):
+
+            if video_obj.file_name is not None:
+
+                # Thumbnails might be in one of two locations
+                thumb_path = utils.find_thumbnail_webp(self.app_obj, video_obj)
+
+                # If the video's parent container has an alternative download
+                #   destination set, we must check the corresponding media
+                #   data object. If the latter also has a media.Video object
+                #   matching this video, then this function returns None and
+                #   nothing is deleted
+                if thumb_path is not None:
+
+                    thumb_path = self.check_video_in_actual_dir(
+                        media_data_obj,
+                        video_obj,
+                        thumb_path,
+                    )
+
+                if thumb_path is not None \
+                and os.path.isfile(thumb_path):
+
+                    # Delete the thumbnail file
+                    os.remove(thumb_path)
+                    self.webp_deleted_count += 1
+
+
     def delete_archive(self, media_data_obj):
 
         """Called by self.tidy_directory().
@@ -939,7 +1025,7 @@ class TidyManager(threading.Thread):
         """
 
         if DEBUG_FUNC_FLAG:
-            utils.debug_time('top 942 delete_archive')
+            utils.debug_time('top 1028 delete_archive')
 
         archive_path = os.path.abspath(
             os.path.join(
@@ -974,7 +1060,7 @@ class TidyManager(threading.Thread):
         """
 
         if DEBUG_FUNC_FLAG:
-            utils.debug_time('top 977 call_moviepy')
+            utils.debug_time('top 1063 call_moviepy')
 
         try:
             clip = moviepy.editor.VideoFileClip(video_path)
@@ -984,7 +1070,8 @@ class TidyManager(threading.Thread):
 
             self.app_obj.main_win_obj.output_tab_write_stdout(
                 1,
-                '   Video file might be corrupt: \'' + video_obj.name + '\'',
+                '   ' + _('Video file might be corrupt:') + ' \'' \
+                + video_obj.name + '\'',
             )
 
 
@@ -1018,7 +1105,7 @@ class TidyManager(threading.Thread):
         """
 
         if DEBUG_FUNC_FLAG:
-            utils.debug_time('top 1021 check_video_in_actual_dir')
+            utils.debug_time('top 1108 check_video_in_actual_dir')
 
         if container_obj.dbid == container_obj.master_dbid:
 
@@ -1054,6 +1141,6 @@ class TidyManager(threading.Thread):
         """
 
         if DEBUG_FUNC_FLAG:
-            utils.debug_time('top 1057 stop_tidy_operation')
+            utils.debug_time('top 1144 stop_tidy_operation')
 
         self.running_flag = False
