@@ -26,6 +26,7 @@ from gi.repository import Gtk, Gdk
 
 # Import other modules
 import datetime
+import glob
 import locale
 import math
 import os
@@ -115,7 +116,89 @@ drag_drop_text=None, no_modify_flag=None):
     return None
 
 
-def add_links_to_textview_from_clipboard(app_obj, textview, mark_start=None,
+def add_links_to_textview(app_obj, link_list, textbuffer, mark_start=None,
+mark_end=None, drag_drop_text=None):
+
+    """Called by mainwin.AddVideoDialogue.__init__(),
+    .on_window_drag_data_received() and .clipboard_timer_callback().
+
+    Also called by utils.add_links_to_textview_from_clipboard().
+
+    Function to add valid URLs from the clipboard to a Gtk.TextView, ignoring
+    anything that is not a valid URL, and ignoring duplicate URLs.
+
+    If some text is supplied as an argument, uses that text rather than the
+    clipboard text
+
+    Args:
+
+        app_obj (mainapp.TartubeApp): The main application
+
+        link_list (list): List of URLs to add to the textview
+
+        textbuffer (Gtk.TextBuffer): The textbuffer to which valis URLs should
+            be added (unless they are duplicates)
+
+        mark_start, mark_end (Gtk.TextMark): The marks at the start/end of the
+            buffer (using marks rather than iters prevents Gtk errors)
+
+    """
+
+    # Eliminate empty lines and any lines that are not valid URLs (we assume
+    #   that it's one URL per line)
+    # At the same time, trim initial/final whitespace
+    valid_list = []
+    for line in link_list:
+        if check_url(line):
+
+            line = strip_whitespace(line)
+            if re.search('\S', line):
+                valid_list.append(line)
+
+    if valid_list:
+
+        # Some URLs survived the cull
+
+        # Get the contents of the buffer
+        if mark_start is None or mark_end is None:
+
+            # No Gtk.TextMarks supplied, we're forced to use iters
+            buffer_text = textbuffer.get_text(
+                textbuffer.get_start_iter(),
+                textbuffer.get_end_iter(),
+                # Don't include hidden characters
+                False,
+            )
+
+        else:
+
+            buffer_text = textbuffer.get_text(
+                textbuffer.get_iter_at_mark(mark_start),
+                textbuffer.get_iter_at_mark(mark_end),
+                False,
+            )
+
+        # Remove any URLs that already exist in the buffer
+        line_list = buffer_text.split('\n')
+        mod_list = []
+        for line in valid_list:
+            if not line in line_list:
+                mod_list.append(line)
+
+        # Add any surviving URLs to the buffer, first adding a newline
+        #   character, if the buffer doesn't end in one
+        if mod_list:
+
+            if not re.search('\n\s*$', buffer_text) and buffer_text != '':
+                mod_list[0] = '\n' + mod_list[0]
+
+            textbuffer.insert(
+                textbuffer.get_end_iter(),
+                str.join('\n', mod_list) + '\n',
+            )
+
+
+def add_links_to_textview_from_clipboard(app_obj, textbuffer, mark_start=None,
 mark_end=None, drag_drop_text=None):
 
     """Called by mainwin.AddVideoDialogue.__init__(),
@@ -131,8 +214,8 @@ mark_end=None, drag_drop_text=None):
 
         app_obj (mainapp.TartubeApp): The main application
 
-        textview (Gtk.TextBuffer): The textview to which valis URLs should be
-            added (unless they are duplicates)
+        textbuffer (Gtk.TextBuffer): The textbuffer to which valis URLs should
+            be added (unless they are duplicates)
 
         mark_start, mark_end (Gtk.TextMark): The marks at the start/end of the
             buffer (using marks rather than iters prevents Gtk errors)
@@ -153,59 +236,15 @@ mark_end=None, drag_drop_text=None):
         # Ignore the clipboard, and use the specified text
         cliptext = drag_drop_text
 
-    # Eliminate empty lines and any lines that are not valid URLs (we assume
-    #   that it's one URL per line)
-    # At the same time, trim initial/final whitespace
-    valid_list = []
-    if cliptext is not None and cliptext != Gdk.SELECTION_CLIPBOARD:
-        for line in cliptext.split('\n'):
-            if check_url(line):
-
-                line = strip_whitespace(line)
-                if re.search('\S', line):
-                    valid_list.append(line)
-
-    if valid_list:
-
-        # Some URLs survived the cull
-
-        # Get the contents of the buffer
-        if mark_start is None or mark_end is None:
-
-            # No Gtk.TextMarks supplied, we're forced to use iters
-            buffer_text = textview.get_text(
-                textview.get_start_iter(),
-                textview.get_end_iter(),
-                # Don't include hidden characters
-                False,
-            )
-
-        else:
-
-            buffer_text = textview.get_text(
-                textview.get_iter_at_mark(mark_start),
-                textview.get_iter_at_mark(mark_end),
-                False,
-            )
-
-        # Remove any URLs that already exist in the buffer
-        line_list = buffer_text.split('\n')
-        mod_list = []
-        for line in valid_list:
-            if not line in line_list:
-                mod_list.append(line)
-
-        # Add any surviving URLs to the buffer, first adding a newline
-        #   character, if the buffer doesn't end in one
-        if mod_list:
-
-            if not re.search('\n\s*$', buffer_text) and buffer_text != '':
-                mod_list[0] = '\n' + mod_list[0]
-
-            textview.insert(
-                textview.get_end_iter(),
-                str.join('\n', mod_list) + '\n',
-            )
+    # Pass the text on to the next function, first converting it into a list
+    if cliptext is not None:
+        add_links_to_textview(
+            app_obj,
+            cliptext.split('\n'),
+            textbuffer,
+            mark_start,
+            mark_end,
+        )
 
 
 def check_url(url):
@@ -316,7 +355,8 @@ def convert_path_to_temp(app_obj, old_path, move_flag=False):
 
     Returns:
 
-        new_path: The converted full file path
+        new_path: The converted full file path, or None if a filesystem error
+            occurs
 
     """
 
@@ -327,12 +367,18 @@ def convert_path_to_temp(app_obj, old_path, move_flag=False):
 
     # The destination folder must exist, before moving files into it
     if not os.path.exists(new_dir):
-        os.makedirs(new_dir)
+        try:
+            os.makedirs(new_dir)
+        except:
+            return None
 
     # On MS Windows, a file name new_path must not exist, or an exception will
     #   be raised
     if os.path.isfile(new_path):
-        os.remove(new_path)
+        try:
+            os.remove(new_path)
+        except:
+            return None
 
     # Move the file now, if the calling code requires that
     if move_flag:
@@ -385,6 +431,30 @@ def convert_seconds_to_string(seconds, short_flag=False):
         return str(datetime.timedelta(seconds=seconds))
 
 
+def convert_youtube_id_to_rss(media_type, youtube_id):
+
+    """Can be called by anything; usually called by
+    media.GenericRemoteContainer.set_rss().
+
+    Convert the channel/playlist ID provided by YouTube into the full URL for
+    the channel/playlist RSS feed.
+
+    Args:
+
+        media_type (str): 'channel' or 'playlist'
+
+        youtube_id (str): The YouTube channel or playlist ID
+
+    Return values:
+
+        The full URL for the RSS feed
+
+    """
+
+    return 'https://www.youtube.com/feeds/videos.xml?' + media_type \
+    + '_id=' + youtube_id
+
+
 def convert_youtube_to_hooktube(url):
 
     """Can be called by anything.
@@ -445,6 +515,38 @@ def convert_youtube_to_invidious(url):
     return url
 
 
+def convert_youtube_to_other(app_obj, url):
+
+    """Can be called by anything.
+
+    Converts a YouTube weblink to a weblink pointing at an alternative
+    YouTube front-end (such as Hooktube or Invidious).
+
+    Args:
+
+        app_obj (mainapp.TartubeApp): The main application
+
+        url (str): The weblink to convert
+
+    Returns:
+
+        The converted string
+
+    """
+
+    if re.search(r'^https?:\/\/(www)+\.youtube\.com', url):
+
+        url = re.sub(
+            r'youtube\.com',
+            app_obj.custom_dl_divert_website,
+            url,
+            # Substitute first occurence only
+            1,
+        )
+
+    return url
+
+
 def debug_time(msg):
 
     """Called by all functions in downloads.py, info.py, mainapp.py,
@@ -581,6 +683,11 @@ def find_available_name(app_obj, old_name, min_value=2, max_value=9999):
     slightly modifies the name, converting 'my_name' into 'my_name_N', where N
     is the smallest positive integer for which the name is available.
 
+    If the specified old_name is already in that format (for example,
+    'Channel_4'), then the old number is stripped away, and this function
+    starts looking from the first integer after that (for example,
+    'Channel_5').
+
     To preclude any possibility of infinite loops, the function will give up
     after max_value attempts.
 
@@ -603,6 +710,19 @@ def find_available_name(app_obj, old_name, min_value=2, max_value=9999):
 
     """
 
+    # If old_name is already in the format 'my_name_N', where N is an integer
+    #   in the range min_value < N < max_value, then strip it away
+    if re.search(r'\_\d+$', old_name):
+
+        number = int(re.sub(r'^.*\_(\d+)$', r'\1', old_name))
+        mod_name = re.sub(r'^(.*)\_\d+$', r'\1', old_name)
+
+        if number >= 2 and number < max_value:
+
+            old_name = mod_name
+            min_value = number + 1
+
+    # Find an available name
     if max_value != -1:
 
         for n in range (min_value, max_value):
@@ -652,7 +772,7 @@ def find_thumbnail(app_obj, video_obj, temp_dir_flag=False):
 
     """
 
-    for ext in ('.jpg', '.png', '.gif'):
+    for ext in formats.IMAGE_FORMAT_LIST:
 
         # Look in Tartube's permanent data directory
         path = video_obj.get_actual_path_by_ext(app_obj, ext)
@@ -672,6 +792,43 @@ def find_thumbnail(app_obj, video_obj, temp_dir_flag=False):
     return None
 
 
+def find_thumbnail_webp(app_obj, video_obj):
+
+    """Called by tidy.TidyManager.delete_webp().
+
+    In June 2020, YouTube started serving .webp thumbnails. At the time of
+    writing (v2.1.027), Gtk can't display them. A youtube-dl fix is expected,
+    which will convert .webp thumbnails to .jpg; in anticipation of that, we
+    add an option to remove .webp files.
+
+    This is a modified version of utils.find_thumbnail(), which looks for
+    thumbnails in the .webp or malformed .jpg format, and return the path to
+    the thumbnail file if one is found.
+
+    Args:
+
+        app_obj (mainapp.TartubeApp): The main application
+
+        video_obj (media.Video): The video object handling the downloaded video
+
+    Returns:
+
+        path (str): The full path to the thumbnail file, or None
+
+    """
+
+    path = video_obj.get_actual_path_by_ext(app_obj, '.webp')
+    if os.path.isfile(path):
+       return path
+
+    # malformed .jpg thumbnail files have an extension .jpg?sqp=-XXX, where XXX
+    #   is a large number of random characters
+    path = video_obj.get_actual_path_by_ext(app_obj, '.jpg?*')
+    for actual_path in glob.glob(path):
+        if os.path.isfile(actual_path):
+            return actual_path
+
+
 def format_bytes(num_bytes):
 
     """Can be called by anything.
@@ -702,7 +859,8 @@ def format_bytes(num_bytes):
 
 
 def generate_system_cmd(app_obj, media_data_obj, options_list,
-dl_sim_flag=False, divert_mode=None):
+dl_sim_flag=False, dl_classic_flag=False, missing_video_check_flag=None,
+divert_mode=None):
 
     """Called by downloads.VideoDownloader.do_download() and
     mainwin.SystemCmdDialogue.update_textbuffer().
@@ -725,12 +883,19 @@ dl_sim_flag=False, divert_mode=None):
         dl_sim_flag (bool): True if a simulated download is to take place,
             False if a real download is to take place
 
+        dl_classic_flag (bool): True if the download operation was launched
+            from the Classic Mode Tab, False otherwise
+
+        missing_video_check_flag (bool): True if the download operation is
+            trying to detect missing videos (downloaded by user, but since
+            removed by the creator), False otherwise
+
         divert_mode (str): If not None, should be one of the values of
-            mainapp.TartubeApp.custom_dl_divert_mode: 'default', 'hooktube' or
-            'invidious'. If one of the latter two, a media.Video object whose
-            source URL points to YouTube should be converted to HookTube or
-            Invidious (no conversion takes place for channels/playlists/
-            folders)
+            mainapp.TartubeApp.custom_dl_divert_mode: 'default', 'hooktube',
+            'invidious' or 'other'. If not 'default', a media.Video object
+            whose source URL points to YouTube should be converted to the
+            specified alternative YouTube front-end (no conversion takes place
+            for channels/playlists/folders)
 
     Returns:
 
@@ -744,31 +909,48 @@ dl_sim_flag=False, divert_mode=None):
     if dl_sim_flag:
         options_list.append('--dump-json')
 
-    # If actually downloading videos, create an archive file so that, if the
-    #   user deletes the videos, youtube-dl won't try to download them again
-    # (Videos downloaded into a system folder should never create an archive
-    #   file)
-    if app_obj.allow_ytdl_archive_flag \
+    # If actually downloading videos, use (or create) an archive file so that,
+    #   if the user deletes the videos, youtube-dl won't try to download them
+    #   again
+    # We don't use an archive file when:
+    #   1. Downloading into a system folder
+    #   2. Checking for missing videos
+    if not missing_video_check_flag \
     and (
-        not isinstance(media_data_obj, media.Folder)
-        or not media_data_obj.fixed_flag
-    ) and (
-        not isinstance(media_data_obj, media.Video)
-        or not isinstance(media_data_obj.parent_obj, media.Folder)
-        or not media_data_obj.parent_obj.fixed_flag
+        not dl_classic_flag and app_obj.allow_ytdl_archive_flag \
+        or dl_classic_flag and app_obj.classic_ytdl_archive_flag
     ):
-        # (Create the archive file in the media data object's default
-        #   sub-directory, not the alternative download destination, as this
-        #   helps youtube-dl to work the way we want it to work)
-        if isinstance(media_data_obj, media.Video):
-            dl_path = media_data_obj.parent_obj.get_default_dir(app_obj)
-        else:
-            dl_path = media_data_obj.get_default_dir(app_obj)
+        if not dl_classic_flag \
+        and (
+            not isinstance(media_data_obj, media.Folder)
+            or not media_data_obj.fixed_flag
+        ) and (
+            not isinstance(media_data_obj, media.Video)
+            or not isinstance(media_data_obj.parent_obj, media.Folder)
+            or not media_data_obj.parent_obj.fixed_flag
+        ):
+            # (Create the archive file in the media data object's default
+            #   sub-directory, not the alternative download destination, as
+            #   this helps youtube-dl to work the way we want it to work)
+            if isinstance(media_data_obj, media.Video):
+                dl_path = media_data_obj.parent_obj.get_default_dir(app_obj)
+            else:
+                dl_path = media_data_obj.get_default_dir(app_obj)
 
-        options_list.append('--download-archive')
-        options_list.append(
-            os.path.abspath(os.path.join(dl_path, 'ytdl-archive.txt')),
-        )
+            options_list.append('--download-archive')
+            options_list.append(
+                os.path.abspath(os.path.join(dl_path, 'ytdl-archive.txt')),
+            )
+
+        elif dl_classic_flag:
+
+            # Create the archive file in destination directory
+            dl_path = media_data_obj.dummy_dir
+
+            options_list.append('--download-archive')
+            options_list.append(
+                os.path.abspath(os.path.join(dl_path, 'ytdl-archive.txt')),
+            )
 
     # Show verbose output (youtube-dl debugging mode), if required
     if app_obj.ytdl_write_verbose_flag:
@@ -778,15 +960,19 @@ dl_sim_flag=False, divert_mode=None):
     #   user has provided one
     if app_obj.ffmpeg_path is not None:
         options_list.append('--ffmpeg-location')
-        options_list.append('"' + app_obj.ffmpeg_path + '"')
+        options_list.append(app_obj.ffmpeg_path)
 
-    # Convert a YouTube URL to HookTube/Invidious, if required
+    # Convert a YouTube URL to an alternative YouTube front-end, if required
     source = media_data_obj.source
     if isinstance(media_data_obj, media.Video) and divert_mode:
         if divert_mode == 'hooktube':
             source = convert_youtube_to_hooktube(source)
         elif divert_mode == 'invidious':
             source = convert_youtube_to_invidious(source)
+        elif divert_mode == 'custom' \
+        and app_obj.custom_dl_divert_website is not None \
+        and len(app_obj.custom_dl_divert_website) > 2:
+            source = convert_youtube_to_other(app_obj, source)
 
     # Convert a path beginning with ~ (not on MS Windows)
     ytdl_path = app_obj.ytdl_path
